#ifndef BASE_CONTROLLER_HPP
#define BASE_CONTROLLER_HPP

#include <ros/ros.h>
#include <std_msgs/Bool.h>
#include <geometry_msgs/Pose.h>
#include <geometry_msgs/Twist.h>
#include <Eigen/Dense>

class BaseController
{
public:
    BaseController(ros::NodeHandle& node_handle);
    void update();
    
private:
    void target_pose_callback(const geometry_msgs::Pose& msg);
    void target_twist_callback(const geometry_msgs::Twist& msg);
    void feedback_pose_callback(const geometry_msgs::Pose& msg);
    void planning_status_callback(const std_msgs::Bool& msg);
    Eigen::MatrixXd compute_jacobian(double x_r, double y_r, double phi);
    void compute_twist();
    void reset_twist();

    ros::NodeHandle node_handle_;
    ros::Publisher husky_vel_publisher_;
    ros::Subscriber target_twist_subscriber_;
    ros::Subscriber target_pose_subscriber_;
    ros::Subscriber feedback_pose_subscriber_;
    ros::Subscriber planning_status_subscriber_;
    geometry_msgs::Twist husky_vel_msg_;

    Eigen::Vector2d current_position_;
    Eigen::Vector2d target_position_;
    Eigen::Vector2d target_velocity_;

    double max_linear_velocity_;
    double max_angular_velocity_;
    double current_heading_;
    bool is_planning_done_;
};

#endif // BASE_CONTROLLER_HPP