#ifndef CUBIC_ACTION_SERVER_HPP
#define CUBIC_ACTION_SERVER_HPP

#include <ros/ros.h>
#include <actionlib/server/simple_action_server.h>
#include <highlevel_msgs/PoseCommandAction.h>
#include <geometry_msgs/Pose.h>
#include <geometry_msgs/Twist.h>
#include <Eigen/Dense>
#include <tf2_geometry_msgs/tf2_geometry_msgs.h>


class CubicActionServer {
public:
	CubicActionServer(ros::NodeHandle& node_handle);
	void update();

private:
	bool load_params();
	void cubic_action_callback(const highlevel_msgs::PoseCommandGoalConstPtr& goal);
	void pose_fbk_callback(const geometry_msgs::Pose& pose_msg);
	void compute_trajectory_point();

	ros::NodeHandle& node_handle_;
	ros::Subscriber pose_fbk_subscriber_;
	ros::Publisher pose_ref_publisher_;
    ros::Publisher twist_ref_publisher_;

    geometry_msgs::Pose pose_cmd_;
    geometry_msgs::Twist twist_cmd_;
	highlevel_msgs::PoseCommandGoal action_goal_;
	highlevel_msgs::PoseCommandFeedback action_feedback_;
	actionlib::SimpleActionServer<highlevel_msgs::PoseCommandAction> action_server_;
    tf2::Quaternion target_quat_;
    tf2::Quaternion fbk_quat_;

	Eigen::VectorXd target_position_;
    Eigen::VectorXd initial_position_;
    Eigen::VectorXd ground_truth_position_;
    Eigen::VectorXd next_position_;
    Eigen::VectorXd next_velocity_;
    Eigen::Quaterniond target_ori_;
    Eigen::Quaterniond initial_ori_;
    Eigen::Quaterniond ground_truth_ori_;
    Eigen::Quaterniond next_ori_;

	std::string pose_subscriber_topic_ ;
	double loop_rate_;
	double target_T_;
    double initial_T_;
    double z_min_;
    double a0_;
    double a1_;
    double a2_;
    double a3_;
    double s_;
    double s_dot_;
    bool is_initialised_;
    bool is_pose_available_;
};

#endif