#ifndef CUBIC_POLYNOMIAL_PLANNER_H
#define CUBIC_POLYNOMIAL_PLANNER_H

#include <ros/ros.h>
#include <std_msgs/Bool.h>
#include <geometry_msgs/Pose.h>
#include <geometry_msgs/Twist.h>
#include <highlevel_msgs/MoveTo.h>
#include <Eigen/Dense>
#include <string>
#include <tf2_geometry_msgs/tf2_geometry_msgs.h>

class CubicPolynomialPlanner
{
public:
    CubicPolynomialPlanner(ros::NodeHandle& node_handle);
    void update();
    
private:
    bool load_params();
    bool move_to_callback(highlevel_msgs::MoveTo::Request& req, highlevel_msgs::MoveTo::Response& resp);
    bool move_ori_callback(highlevel_msgs::MoveTo::Request& req, highlevel_msgs::MoveTo::Response& resp);
    void pose_callback(const geometry_msgs::Pose& msg);
    void compute_trajectory_point();

    ros::NodeHandle node_handle_;
    ros::ServiceServer move_to_server_;
    ros::ServiceServer move_ori_server_;
    ros::Subscriber pose_subscriber_;
    ros::Publisher pose_publisher_;
    ros::Publisher twist_publisher_;
    ros::Publisher planning_status_publisher_;
    geometry_msgs::Pose pose_cmd_;
    geometry_msgs::Twist twist_cmd_;
    std_msgs::Bool planning_status_msg_;
    
    Eigen::VectorXd target_position_;
    Eigen::VectorXd initial_position_;
    Eigen::VectorXd ground_truth_position_;
    Eigen::VectorXd next_position_;
    Eigen::VectorXd next_velocity_;
    Eigen::Quaterniond target_ori_;
    Eigen::Quaterniond initial_ori_;
    Eigen::Quaterniond ground_truth_ori_;
    Eigen::Quaterniond next_ori_;

    std::string pose_subscriber_topic_;
    double target_T_;
    double initial_T_;
    double z_min_;
    double a0_;
    double a1_;
    double a2_;
    double a3_;
    double s_;
    double s_dot_;
    bool is_initialised_;
    bool is_pose_available_;
};

#endif