#ifndef JOINT_CONTROLLER_HPP
#define JOINT_CONTROLLER_HPP

#include <pinocchio/multibody/model.hpp>
#include <pinocchio/multibody/data.hpp>
#include <pinocchio/parsers/urdf.hpp>
#include <pinocchio/algorithm/compute-all-terms.hpp>
#include <ros/ros.h>
#include <sensor_msgs/JointState.h>
#include <std_msgs/Bool.h>
#include <std_msgs/Float64MultiArray.h>
#include <Eigen/Dense>
#include <string>

class JointController
{
public:
    JointController(ros::NodeHandle& node_handle);
    void update();

private:
    bool load_params();
    void joint_states_callback(const sensor_msgs::JointState& msg);
    void position_cmd_callback(const std_msgs::Float64MultiArray& msg);
    void velocity_cmd_callback(const std_msgs::Float64MultiArray& msg);
    void planning_status_callback(const std_msgs::Bool& msg);
    void compute_joint_torques();

    ros::NodeHandle node_handle_;
    ros::Publisher joint_torque_publisher_;
    ros::Subscriber joint_states_subscriber_;
    ros::Subscriber velocity_cmd_subscriber_;
    ros::Subscriber position_cmd_subscriber_;
    ros::Subscriber planning_status_subscriber_;
    ros::ServiceClient trigger_client_;
    std_msgs::Float64MultiArray joint_torque_cmd_;

    pinocchio::Model robot_model_;
    pinocchio::Data robot_data_;
    Eigen::MatrixXd M_; // Mass matrix
    Eigen::MatrixXd h_; 
    Eigen::VectorXd tau_cmd_;
    Eigen::VectorXd q_fbk_;
    Eigen::VectorXd qd_fbk_;
    Eigen::VectorXd q_ref_;
    Eigen::VectorXd qd_ref_;
    Eigen::VectorXd qdd_ref_;
    Eigen::VectorXd qdd_cmd_; // Commanded joint accelerations
    
    std::string urdf_path_;
    std::string joint_states_topic_;
    std::string torque_cmd_topic_;
    int n_joints_;
    double loop_rate_;
    double loop_time_;
    double K_;
    double D_;
    bool is_joint_detected_;
    bool is_target_reached_;
    bool is_zero_vel_;
};

#endif // JOINT_CONTROLLER_HPP