#ifndef POSE_ACTION_CLIENT_HPP
#define POSE_ACTION_CLIENT_HPP

#include <ros/ros.h>
#include <Eigen/Dense>
#include <actionlib/client/simple_action_client.h>
#include <highlevel_msgs/PoseCommandAction.h>

class PoseActionClient {
public:
    PoseActionClient(ros::NodeHandle& node_handle);
    void update();

protected:
    void active_callback();
    // void feedback_callback(const highlevel_msgs::PoseCommandFeedbackConstPtr& feedback);
    void feedback_callback(const highlevel_msgs::PoseCommandFeedbackConstPtr& feedback);
    void done_callback(const actionlib::SimpleClientGoalState& state, const highlevel_msgs::PoseCommandResultConstPtr& result);

private:
    bool load_params();
    
    ros::NodeHandle& node_handle_;
    ros::Subscriber feedback_subscriber_;
    highlevel_msgs::PoseCommandGoal action_goal_;
    actionlib::SimpleActionClient<highlevel_msgs::PoseCommandAction> action_client_;

    Eigen::MatrixXd target_translations_;
    Eigen::MatrixXd target_orientations_;
    Eigen::VectorXd target_Ts_;

    std::string feedback_subscriber_topic_;
    int n_targets_;
    int target_index_;
    double distance_translation_;
    double distance_orientation_;
    double time_elapsed_;
};

#endif