#include "keyboard_controller/KeyboardController.hpp"

KeyboardController::KeyboardController(ros::NodeHandle& node_handle) : node_handle_(node_handle)
{
    teleop_subscriber_ = node_handle_.subscribe("/teleop/cmd", 1, &KeyboardController::teleop_callback, this);
    husky_vel_publisher_ = node_handle_.advertise<geometry_msgs::Twist>("/husky_velocity_controller/cmd_vel", 1, this);
}

void KeyboardController::update()
{
    husky_vel_publisher_.publish(husky_vel_cmd_);
    reset_vel();
}

void KeyboardController::teleop_callback(const std_msgs::String& msg)
{
    switch(msg.data.c_str()[0])
    {
        case 'i':
            ROS_INFO("Moving forward");
            husky_vel_cmd_.linear.x = 0.5;
            husky_vel_cmd_.angular.z = 0.0;
            break;
        case 'u':
            ROS_INFO("Turning left");
            husky_vel_cmd_.linear.x = 0.5;
            husky_vel_cmd_.angular.z = 0.5;
            break;
        case 'o':
            ROS_INFO("Turning right");
            husky_vel_cmd_.linear.x = 0.5;
            husky_vel_cmd_.angular.z = -0.5;
            break;
        default:
            ROS_INFO("Keypress not recognized");
    }
}

void KeyboardController::reset_vel()
{
    husky_vel_cmd_.linear.x = 0;
    husky_vel_cmd_.angular.z = 0;
}
