#ifndef JOINT_CONTROLLER_HPP
#define JOINT_CONTROLLER_HPP

#include <ros/ros.h>
#include <std_msgs/Bool.h>
#include <std_msgs/Float64MultiArray.h>
#include <Eigen/Dense>
#include <string>

class JointController
{
public:
    JointController(ros::NodeHandle& node_handle);
    void update();

private:
    bool load_params();
    void position_cmd_callback(const std_msgs::Float64MultiArray& msg);
    void velocity_cmd_callback(const std_msgs::Float64MultiArray& msg);
    void planning_status_callback(const std_msgs::Bool& msg);

    ros::NodeHandle node_handle_;
    ros::Publisher joint_velocity_publisher_;
    ros::Publisher joint_position_publisher_;
    ros::Subscriber velocity_cmd_subscriber_;
    ros::Subscriber position_cmd_subscriber_;
    ros::Subscriber planning_status_subscriber_;
    ros::ServiceClient trigger_client_;
    std_msgs::Float64MultiArray joint_velocity_cmd_;
    std_msgs::Float64MultiArray joint_position_cmd_;
    
    std::string position_cmd_topic_;
    std::string velocity_cmd_topic_;
    int n_joints_;
    bool is_target_reached_;
};

#endif // JOINT_CONTROLLER_HPP