#ifndef JOINT_PLANNER_HPP
#define JOINT_PLANNER_HPP

#include <ros/ros.h>
#include <sensor_msgs/JointState.h>
#include <std_msgs/Bool.h>
#include <std_msgs/Float64MultiArray.h>
#include <std_srvs/Trigger.h>
#include <Eigen/Dense>

class JointPlanner
{
public:
    JointPlanner(ros::NodeHandle& node_handle);
    void update();

private:
    bool load_params();
    void compute_field();
    void check_target_reached();
    void joint_states_callback(const sensor_msgs::JointState& msg);
    bool planner_start_callback(std_srvs::TriggerRequest& req, std_srvs::TriggerResponse& resp);

    ros::NodeHandle node_handle_;
    ros::Publisher joint_velocity_publisher_;
    ros::Publisher joint_position_publisher_;
    ros::Publisher planning_status_publisher_;
    ros::Subscriber joint_states_subscriber_;
    ros::ServiceServer planner_start_server_;
    std_msgs::Float64MultiArray joint_velocity_cmd_;
    std_msgs::Float64MultiArray joint_position_cmd_;
    std_msgs::Bool planning_status_msg_;

    Eigen::VectorXd current_joint_positions_;
    Eigen::VectorXd target_joint_positions_;

    std::string joint_states_topic_;
    std::string planner_start_service_;
    std::vector<double> joint_velocity_limits_;
    std::vector<double> joint_target_list_;
    int n_joints_;
    double k_att_;
    double tolerance_;
    double loop_rate_;
    double loop_time_;
    bool is_planner_called_;
};

#endif // JOINT_PLANNER_HPP