#ifndef POTENTIAL_FIELD_PLANNER_HPP
#define POTENTIAL_FIELD_PLANNER_HPP

#include <ros/ros.h>
#include <std_msgs/Bool.h>
#include <geometry_msgs/Twist.h>
#include <geometry_msgs/Pose.h>
#include <std_srvs/Trigger.h>
#include <Eigen/Dense>

class PotentialFieldPlanner
{
public:
    PotentialFieldPlanner(ros::NodeHandle& node_handle);
    void update();

private:
    void pose_callback(const geometry_msgs::Pose& msg);
    bool planner_start_callback(std_srvs::TriggerRequest& req, std_srvs::TriggerResponse& resp);
    bool load_params();
    void check_target_reached();
    void compute_field();

    ros::NodeHandle node_handle_;
    ros::Subscriber pose_subscriber_;
    ros::ServiceServer start_planning_server_;
    ros::Publisher pose_publisher_;
    ros::Publisher twist_publisher_;
    ros::Publisher planning_status_publisher_;
    
    std_msgs::Bool planning_status_msg_;
    geometry_msgs::Twist twist_msg_;
    geometry_msgs::Pose pose_msg_;

    Eigen::Vector2d target_position_;
    Eigen::Vector2d current_position_;

    bool is_planner_called_;
    double k_att_;
    double max_linear_velocity_;
    double max_angular_velocity_;
    double loop_time_;
    double loop_rate_;
};

#endif // POTENTIAL_FIELD_PLANNER_HPP